// Copyright (c) 2003 Charles Samuels <charles@kde.org>
// See the file COPYING for redistribution terms.

#include "oblique.h"
#include "base.h"
#include "tree.h"
#include "file.h"
#include "selector.h"
#include "cmodule.h"

#include <kconfig.h>
#include <kstddirs.h>

extern "C" Plugin *create_plugin()
{
	return new Oblique();
}



Oblique::Oblique()
	: Playlist(0, 0), mVisible(false), mSchemaCollection("oblique/schemas")
{
	KConfigGroup g(KGlobal::config(), "oblique");

	mFileOfQuery = g.readEntry("schema", "standard");
	loadQuery(mFileOfQuery);

	mBase = new Base(::locate("data", "noatun/")+"/oblique-list");
	mView = new Tree(&mQuery);
	mSelector = new SequentialSelector(mView);

	new Configure(this);

	mView->connect(mBase, SIGNAL(added(File)), SLOT(insert(File)));
	mView->connect(mBase, SIGNAL(removed(File)), SLOT(remove(File)));

	FileId high = mBase->high();
	// get more pot until we are high
	for (FileId pot=1; pot <= high; pot++)
	{
		File f = mBase->find(pot);
		if (f)
			mView->insert(f);
	}

}

void Oblique::loadQuery(const QString &name)
{
	mFileOfQuery = name;
	mQuery.load(schemaCollection().file(name, false));
}

Oblique::~Oblique()
{
	delete mView;
	delete mBase;
}

void Oblique::reset()
{

}

void Oblique::clear()
{

}

void Oblique::addFile(const KURL &url, bool play)
{
	(void)play; // TODO
	mBase->add(url.path());
}


PlaylistItem Oblique::next()
{
	return mSelector->next();
}

PlaylistItem Oblique::previous()
{
	return mSelector->previous();
}


PlaylistItem Oblique::current()
{
	return mSelector->current();
}

void Oblique::setCurrent(const PlaylistItem &item)
{
	mSelector->setCurrent(static_cast<Item*>(const_cast<PlaylistItemData*>(item.data())));
}


PlaylistItem Oblique::getFirst() const
{
	return 0;
}


PlaylistItem Oblique::getAfter(const PlaylistItem &) const
{
	return 0;
}

bool Oblique::listVisible() const
{
	return mVisible;
}

void Oblique::showList()
{
	mVisible=true;
	mView->show();
}

void Oblique::hideList()
{
	mVisible=false;
	mView->hide();
}

