// Copyright (c) 2003 Charles Samuels <charles@kde.org>
// See the file COPYING for redistribution terms.

#ifndef QUERY_H
#define QUERY_H

#include "base.h"

#include <qregexp.h>
#include <qstring.h>

class Query;
class QDomElement;

class QueryGroup
{
	friend class Query;

	QueryGroup *mFirstChild;
	QueryGroup *mNextSibling;

	int mFuzzyness;
	int mOptions;

	QString mPropertyName;
	QString mPresentation;
	QRegExp mValue;

public:
	QueryGroup();
	QueryGroup(const QueryGroup &copy);
	QueryGroup &operator =(const QueryGroup &copy);


	/**
	 * delete my first child, and my next sibling
	 **/
	~QueryGroup();
	void setFirstChild(QueryGroup *g) { mFirstChild = g; }
	void setNextSibling(QueryGroup *g) { mNextSibling = g; }


	QueryGroup *firstChild() { return mFirstChild; }
	const QueryGroup *firstChild() const { return mFirstChild; }
	QueryGroup *lastChild();
	QueryGroup *nextSibling() { return mNextSibling; }
	const QueryGroup *nextSibling() const { return mNextSibling; }

	QString propertyName() const { return mPropertyName; }
	QRegExp value() const { return mValue; }
	QString presentation() const { return mPresentation; }

	void setPropertyName(const QString &v) { mPropertyName = v; }
	void setPresentation(const QString &v) { mPresentation = v; }
	void setValue(const QRegExp &v) { mValue = v; }

	enum Fuzzyness
	{
		Case = 1<<0, Spaces = 1<<1, Articles = 1<<2, Symbols = 1<<3
	};

	bool fuzzyness(Fuzzyness f) const;

	enum Option
	{
		AutoHide = 1<<0, Disabled = 1<<1, StopOnHit = 1<<2, Unique = 1<<3
	};

	bool option(Option option) const;
	void setOption(Option option, bool on);

	/**
	 * @return if I match @p file
	 **/
	bool matches(const File &file) const;

	QString presentation(const File &file) const;

private:

};




/**
 * a query is the tree structure that is shown to the user
 **/
class Query
{
	QueryGroup *mGroupFirst;

public:
	Query();
	Query(const Query &copy);
	~Query();

	Query &operator =(const Query &copy);

	QueryGroup *firstChild();
	const QueryGroup *firstChild() const;

	void clear();

	/**
	 * @returns the name of the query
	 **/
	QString load(QDomElement element);
	QString load(const QString &filename);
	void save(const QString &name, QDomElement &element);
	void save(const QString &name, const QString &filename);


private:
	void loadGroup(QDomElement element, QueryGroup *parent=0);
	void saveGroup(QDomElement parent, QueryGroup *group);

	void deepCopy(const QueryGroup *from, QueryGroup *toParent);
};



#endif
