// Copyright (c) 2003 Charles Samuels <charles@kde.org>
// See the file COPYING for redistribution terms.

#ifndef TREE_H
#define TREE_H

#include <qwidget.h>
#include <klistview.h>

#include "base.h"

class QueryGroup;
class Query;
class Tree;

class TreeItem : public KListViewItem
{
	QueryGroup *mGroup;
	File mFile;

public:
	TreeItem(Tree *parent, QueryGroup *group, const File &file, const QString &p=0);
	TreeItem(TreeItem *parent, QueryGroup *group, const File &file, const QString &p=0);

	QueryGroup *group() { return mGroup; }
	TreeItem *parent() { return static_cast<TreeItem*>(KListViewItem::parent()); }
	Tree *tree();
	TreeItem *firstChild() { return static_cast<TreeItem*>(KListViewItem::firstChild()); }
	TreeItem *nextSibling() { return static_cast<TreeItem*>(KListViewItem::nextSibling()); }

	File file() { return mFile; }
	void setFile(File file) { mFile = file; }

	TreeItem *find(File item);

	bool playable() const;

	/**
	 * get the next item that is playable logically.
	 * that is, if it has a File, and its parent hasn't a
	 * File
	 **/
	TreeItem *nextPlayable();

	void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align);

	virtual int compare(QListViewItem * i, int col, bool) const;

private:
	TreeItem *next();

};


class Tree : public KListView
{
Q_OBJECT
	Query *mQuery;
	TreeItem *mCurrent;

public:
	Tree(Query *query);
	TreeItem *firstChild();
	TreeItem *find(File item);
	TreeItem *current() { return mCurrent; }

//	bool contains(TreeItem *item) const;

public slots:
	void insert(File file);
	void remove(File file);
	void setCurrent(TreeItem *cur);

private slots:
	void contextMenu(KListView* l, QListViewItem* i, const QPoint& p);

private:
	/**
	 * check if it fits into the group, and create
	 * the tree nodes for it
	 **/
	bool collate(QueryGroup *group, const File &file, TreeItem *childOf=0);

	TreeItem *node(QueryGroup *group, const File &file, TreeItem *childOf);

	/**
	 * remove the siblings and children of the treeitem
	 **/
	void remove(TreeItem *, const File &file);
};


#endif
